Index: dev/etherswitch/ip17x/ip175c.c
===================================================================
--- dev/etherswitch/ip17x/ip175c.c	(revision 250901)
+++ dev/etherswitch/ip17x/ip175c.c	(working copy)
@@ -147,9 +147,9 @@
 	memset(vlans, 0, sizeof(vlans));
 	for (i = 0; i < IP17X_MAX_VLANS; i++) {
 		v = &sc->vlan[i];
-		if (v->vlanid == 0)
+		if ((v->vlanid & ETHERSWITCH_VID_VALID) == 0)
 			continue;
-		vlans[v->vlanid] = v->ports;
+		vlans[v->vlanid & ETHERSWITCH_VID_MASK] = v->ports;
 	}
 
 	for (j = 0, i = 1; i <= IP17X_MAX_VLANS / 2; i++) {
Index: dev/etherswitch/ip17x/ip175d.c
===================================================================
--- dev/etherswitch/ip17x/ip175d.c	(revision 250901)
+++ dev/etherswitch/ip17x/ip175d.c	(working copy)
@@ -94,7 +94,8 @@
 		striptag[i] = 0;
 
 		v = &sc->vlan[i];
-		if (v->vlanid == 0 || sc->vlan_mode == 0) {
+		if ((v->vlanid & ETHERSWITCH_VID_VALID) == 0 ||
+		    sc->vlan_mode == 0) {
 			/* Vlangroup disabled.  Reset the filter. */
 			ip17x_writephy(sc->sc_dev, 22, 14 + i, i + 1);
 			ports[i] = 0x3f;
@@ -105,7 +106,8 @@
 		ports[i] = v->ports;
 
 		/* Setup the filter, write the VLAN id. */
-		ip17x_writephy(sc->sc_dev, 22, 14 + i, v->vlanid);
+		ip17x_writephy(sc->sc_dev, 22, 14 + i,
+		    v->vlanid & ETHERSWITCH_VID_MASK);
 
 		for (j = 0; j < MII_NPHY; j++) {
 			if ((ports[i] & (1 << j)) == 0)
Index: dev/etherswitch/ip17x/ip17x_vlans.c
===================================================================
--- dev/etherswitch/ip17x/ip17x_vlans.c	(revision 250901)
+++ dev/etherswitch/ip17x/ip17x_vlans.c	(working copy)
@@ -74,7 +74,7 @@
 			if (((1 << phy) & sc->phymask) == 0)
 				continue;
 			v = &sc->vlan[i];
-			v->vlanid = i++;
+			v->vlanid = i++ | ETHERSWITCH_VID_VALID;
 			v->ports = (1 << sc->cpuport);
 			for (j = 0; j < MII_NPHY; j++) {
 				if (((1 << j) & sc->phymask) == 0)
@@ -90,10 +90,10 @@
 		 * members of vlan 1.
 		 */
 		v = &sc->vlan[0];
-		v->vlanid = 1;
-		/* Set PVID for everyone. */
+		v->vlanid = 1 | ETHERSWITCH_VID_VALID;
+		/* Set PVID to 1 for everyone. */
 		for (i = 0; i < sc->numports; i++)
-			sc->pvid[i] = v->vlanid;
+			sc->pvid[i] = 1;
 		for (i = 0; i < MII_NPHY; i++) {
 			if ((sc->phymask & (1 << i)) == 0)
 				continue;
@@ -148,11 +148,29 @@
 		return (EINVAL);
 
 	/* IP175C don't support VLAN IDs > 15. */
-	if (IP17X_IS_SWITCH(sc, IP175C) && vg->es_vid > IP175C_LAST_VLAN)
+	if (IP17X_IS_SWITCH(sc, IP175C) &&
+	    (vg->es_vid & ETHERSWITCH_VID_MASK) > IP175C_LAST_VLAN)
 		return (EINVAL);
 
 	/* Vlan ID. */
-	sc->vlan[vg->es_vlangroup].vlanid = vg->es_vid;
+	if (sc->vlan_mode == ETHERSWITCH_VLAN_DOT1Q) {
+		for (i = 0; i < sc->info.es_nvlangroups; i++) {
+			/* Is this Vlan ID already set in another vlangroup ? */
+			if (i != vg->es_vlangroup &&
+			    sc->vlan[i].vlanid & ETHERSWITCH_VID_VALID &&
+			    (sc->vlan[i].vlanid & ETHERSWITCH_VID_MASK) ==
+			    (vg->es_vid & ETHERSWITCH_VID_MASK))
+				return (EINVAL);
+		}
+		sc->vlan[vg->es_vlangroup].vlanid = vg->es_vid &
+		    ETHERSWITCH_VID_MASK;
+		/* Setting the vlanid to zero disables the vlangroup. */
+		if (sc->vlan[vg->es_vlangroup].vlanid == 0) {
+			sc->vlan[vg->es_vlangroup].ports = 0;
+			return (sc->hal.ip17x_hw_setup(sc));
+		}
+		sc->vlan[vg->es_vlangroup].vlanid |= ETHERSWITCH_VID_VALID;
+	}
 
 	/* Member Ports. */
 	sc->vlan[vg->es_vlangroup].ports = 0;
